//****************************************************************************
//Copyright (C) 2008 Airtight Interactive. All Rights Reserved.
//Usage of this code is subject to the Terms of Use accompanying
//this product.
//****************************************************************************

import mx.transitions.Tween;
import mx.utils.Delegate;
import com.airtightinteractive.apps.viewers.simpleViewer.Options;


/**
 * Rollover button. Button shifts on press/release. Used for image and thumbnail navigation buttons.
 * @author Felix Turner
 */
class com.airtightinteractive.apps.viewers.simpleViewer.RolloverButton {
	
	private static var sFlashAlpha:Number = 60;
	private static var sRollOutAlpha:Number = 0;
	
	private var mEnabled:Boolean;
	private var mClickShift:Number;	
	private var mTweenLen:Number;
	private var mHideOnRollOut:Boolean;
	private var mFadeTwn:Tween;
	
	private var mClip_mc:MovieClip;	
	private var mHit_mc:MovieClip;
	private var mBtn_mc:MovieClip;
	
	function RolloverButton(target:MovieClip){
				
		mClip_mc = target;
		mClip_mc.onRollOver = Delegate.create(this,onRollOver);
		mClip_mc.onRollOut = Delegate.create(this,onRollOut);
		mClip_mc.onPress = Delegate.create(this,onPress);
		mClip_mc.onRelease = Delegate.create(this,onRelease);
		mClip_mc.onReleaseOutside = Delegate.create(this,onReleaseOutside);
		
		mHit_mc = mClip_mc.mcHitArea;
		mBtn_mc = mClip_mc.mcArrow;
		mClip_mc.useHandCursor = false;	
		mClip_mc.hitArea = mHit_mc;
		mHit_mc._visible = false;
		
		mClickShift = 0;
		mTweenLen = 5;
		mHideOnRollOut = true;
		mEnabled = true;  
		
		mFadeTwn = new Tween(mClip_mc, "_alpha", null, 0, 0, mTweenLen, false);	
					
	}
	
	private function onRollOver(){
		if (!mEnabled) return;	
		if (mHideOnRollOut){	
			mFadeTwn.continueTo(100,mTweenLen);
		}else{
			doFlash();	
		}
	}
	
	private function onRollOut(){
		if (!mEnabled) return;		
		if (mHideOnRollOut){	
			mFadeTwn.continueTo(sRollOutAlpha,mTweenLen);
		}		
	}
	
	private function doFlash(){		
		mFadeTwn.continueTo(sFlashAlpha,1);
		mFadeTwn.fforward();
		mFadeTwn.continueTo(100,10);				
	}
	
	public function onPress(){
		if (!mEnabled) return;
		
		mClip_mc._x += mClickShift;
		mClip_mc._y += mClickShift;		
	}
	
	public function onRelease(){
		if (!mEnabled) return;
		mClip_mc._x -= mClickShift;
		mClip_mc._y -= mClickShift;
		doAction();				
	}	
	
	public function onReleaseOutside(){
		if (!mEnabled) return;
		mClip_mc._x -= mClickShift;
		mClip_mc._y -= mClickShift;
	}	
	
	public function doAction(){			
	}
	
	public function setPosn(x:Number,y:Number):Void{
		mClip_mc._x = x;
		mClip_mc._y = y;				
	}	
		
	public function setBtnPosn(x:Number,y:Number):Void{		
		mBtn_mc._x = Math.round(x);
		mBtn_mc._y = Math.round(y);				
	}
	
	public function setBtnSize(w:Number,h:Number):Void{
		mBtn_mc._height = Math.round(h);
		mBtn_mc._width = Math.round(w);						
	}
	
	public function setHitPosn(x:Number,y:Number):Void{
		mHit_mc._x = Math.round(x);
		mHit_mc._y = Math.round(y);				
	}
	
	public function setHitSize(w:Number,h:Number):Void{
		mHit_mc._height = Math.round(h);
		mHit_mc._width = Math.round(w);				
	}	
	
	public function get width():Number{
		return mClip_mc._width; 	
	}
	
	public function get height():Number{
		return mClip_mc._height; 	
	}
		
	public function get btnWidth():Number{
		return mBtn_mc._width; 	
	}
	
	public function get btnHeight():Number{
		return mBtn_mc._height; 	
	}
		
	public function set clickShift(a:Number){
		mClickShift = a;
	}
	
	public function get clickShift():Number{
		return mClickShift;
	}
	
	public function set visible(vis:Boolean){		
		mClip_mc._visible = vis;				
	}
	
	public function set hideOnRollOut(b:Boolean){		
		mHideOnRollOut = b;	
		if (!b)	mFadeTwn.continueTo(100,mTweenLen);
	}
	
	public function setRGB(c:Number){		
		var col = new Color(mBtn_mc);
		col.setRGB(c);		
	}
	
	public function set buttonFrame(f:Number){
		mClip_mc.mcArrow.gotoAndStop(f);
	}
	
}