//****************************************************************************
//Copyright (C) 2008 Airtight Interactive. All Rights Reserved.
//Usage of this code is subject to the Terms of Use accompanying
//this product.
//****************************************************************************

import mx.transitions.Tween;
import mx.utils.Delegate;

import com.airtightinteractive.apps.viewers.simpleViewer.ImageArea;
import com.airtightinteractive.apps.viewers.simpleViewer.Options;
import com.airtightinteractive.apps.viewers.simpleViewer.StageManager;
import com.airtightinteractive.apps.viewers.simpleViewer.ThumbArea;
import com.airtightinteractive.apps.viewers.simpleViewer.XMLManager;
import com.airtightinteractive.utils.RectUtil;

/**
 * One Image instance for each large image that is loaded. 
 * Handles Image loading + Transitions + Frame drawing.
 * @author Felix Turner
 */

class com.airtightinteractive.apps.viewers.simpleViewer.Image {
	
	private var mStageManager:StageManager;	
	private var mThumbArea: ThumbArea;
	private var mXMLManager:XMLManager;
	private var mImageArea:ImageArea;
		
	private var mIndex:Number;
	private var mJPGWidth:Number; //orig width of loaded jpg
	private var mJPGHeight:Number;//orig height of loaded jpg	
	public var loaded:Boolean;
	private var mImageURL:String;
	private var mImageLoadError:Boolean;	
	private var mLoadNextonDone:Boolean; //whether to start loading another image after this image has loaded
	
	private var mFadeTween;//:Tween; - don't explicitly type to avoid compile errors in MX 04
	private var mLoader_mcl:MovieClipLoader;
			
	private var mClip_mc:MovieClip;
	private var mLoader_mc:MovieClip;
	private var mFrame_mc:MovieClip;
	private var mImgLoadStatus_mc:MovieClip;	
	
	function Image(target:MovieClip, index:Number, imageURL:String){
		
		mStageManager = StageManager.getInstance();
		mThumbArea = ThumbArea.getInstance();		
		mXMLManager = XMLManager.getInstance();
		mImageArea = ImageArea.getInstance();		
		mIndex = index;		
		var d = target.getNextHighestDepth();
		mClip_mc = target.createEmptyMovieClip("image"+d,d);		
		loaded = false;
		mImageURL = imageURL;
		
		mJPGWidth = 0;
		mJPGHeight = 0;
						
		//Frame MC
		mFrame_mc = mClip_mc.createEmptyMovieClip("frame",mClip_mc.getNextHighestDepth());						
		//ImgLoadStatus MC
		mImgLoadStatus_mc = mClip_mc.attachMovie("ImageLoadbar","mcloadbar",mClip_mc.getNextHighestDepth());
		var col = new Color(mImgLoadStatus_mc);
		col.setRGB(mXMLManager.imageFrameColor);		
		//Loader MC
		mLoader_mc = mClip_mc.createEmptyMovieClip("loader",mClip_mc.getNextHighestDepth());	
		
		mLoadNextonDone = false;
		mFadeTween = new Tween(mClip_mc,"_alpha",null,100,100,1,false);	
		mClip_mc._visible = false;	
		mImageLoadError = false;					
	}
	
	/**
	 * Called by ImageArea
	 * 
	 * @param loadNextonDone whether to start loading another image after this image has loaded
	 */
	public function loadImage(loadNextonDone:Boolean){
	
		mLoadNextonDone = loadNextonDone;
		
		//start loading image
		mLoader_mcl = new MovieClipLoader();
		mLoader_mcl.addListener(this);		
		mLoader_mcl.loadClip(mImageURL,mLoader_mc);		
		mLoader_mc._visible = false;		
		resizeImage();//center loader
	}
			
	function onLoadProgress (target:MovieClip, bytesLoaded:Number, bytesTotal:Number):Void {
		
		var percent:Number = (bytesLoaded/bytesTotal)*100;
    	//pass to thumb load bar    	
    	mThumbArea.showImageProgress(mIndex,percent);    	
    	//if selected image - show prog bar here
		if (mImageArea.currentImageIndex == mIndex) showImageProgress(percent);
	}

	/**
	 * Called when image has completed loading
	 */
	function onLoadInit(){
		mLoader_mc.forceSmoothing = Options.useSmoothing;
		loaded = true;		
		//mLoader_mc._visible = false;
		mLoader_mcl.removeListener(this);
		mJPGWidth = mLoader_mc._width;
		mJPGHeight = mLoader_mc._height;
		mThumbArea.showImageLoadComplete(mIndex);			
		if (mLoadNextonDone && Options.preloadImages) mImageArea.loadNextImage();		
		mImgLoadStatus_mc._visible = false;
		
	}
	
	function onLoadError(){
		mImageLoadError =  true;		
		//if selected - show error icon
		if (mImageArea.currentImageIndex == mIndex) mImageArea.showImage(mIndex);
		//load next image
		if (mLoadNextonDone && Options.preloadImages) mImageArea.loadNextImage();
	}
	
	function showImage(){
		resizeImage();
		mClip_mc._visible = true;	
		//bring to top
		mClip_mc.swapDepths(999999);
		
		//wait imageFadeLength frames to fade up
		mFadeTween.continueTo(0,1);
		mFadeTween.fforward();
		mFadeTween.continueTo(0,Options.imageFadeLength);
		mFadeTween.onMotionFinished = Delegate.create(this,onFadeWaitDone);	
		if (!Options.captionUnderThumbs) mStageManager.setCaptionVisible(false);
	}
	
	function onFadeWaitDone(){	
		//fade in		
		mFadeTween.continueTo(100,Options.imageFadeLength);
		mFadeTween.onMotionFinished = null;
		mImageArea.fadeInButtons();			
		if (!Options.captionUnderThumbs) mStageManager.setCaptionVisible(true);
	}
	
	function hideImage(){	
		mFadeTween.continueTo(100,1);
		mFadeTween.fforward();
		mFadeTween.continueTo(0,Options.imageFadeLength);
		mFadeTween.onMotionFinished = Delegate.create(this,fadeOutDone);
		mImageArea.fadeOutButtons();
	}
	
	function fadeOutDone(){
		mClip_mc._visible = false;			
		mFadeTween.onMotionFinished = null;
	}
		
	/**
	 * Calculate image and frame sizes based on imageArea size
	 */
	function resizeImage(){
		
		var w,h , fw, imgW, imgH, imgX, imgY, imgAspect, jpgAspect :Number;
		
		fw = mXMLManager.frameWidth;		
		w = mImageArea.width - fw*2;
		h = mImageArea.height - fw*2;	
		
		//center loader
		mImgLoadStatus_mc._x = Math.round((w - mImgLoadStatus_mc._width)/2) + fw;
		mImgLoadStatus_mc._y = Math.round((h - mImgLoadStatus_mc._height)/2) + fw;		
				
		if (mJPGWidth == 0) return; //don't resize until image loaded
		
		//calc frame size		
		imgAspect = w/h;
		jpgAspect = mJPGWidth/mJPGHeight;
		//scale image proportionately
		if (jpgAspect > imgAspect){
			imgW = Math.min(w,mJPGWidth);
			imgH = Math.round(imgW/jpgAspect);			
		}else{
			imgH = Math.min(h,mJPGHeight);
			imgW = Math.round(imgH*jpgAspect);			
		}	
		
		//calc image Y posn
		switch (Options.imageVAlign) { 			
			case "top":
				imgY = fw;
				break;			
			case "bottom":
				imgY =  h-imgH + fw;
				break;				
			default : //center					
				imgY =  Math.round((h-imgH)/2) + fw;													
				break;
		}

		//calc image X posn
		switch (Options.imageHAlign) { 			
			case "left":
				imgX = fw;
				break;			
			case "right":
				imgX =  w-imgW + fw;
				break;			
			default : //center					
				imgX =  Math.round((w-imgW)/2) + fw;													
				break;
		}		
				
		//size + posn jpg				
		mLoader_mc._width = imgW;
		mLoader_mc._height = imgH;							
		mLoader_mc._x = imgX;
		mLoader_mc._y  = imgY;
				
		//draw frame
		mFrame_mc.clear();
		mFrame_mc.beginFill(mXMLManager.imageFrameColor, 100);
		RectUtil.hollowRectangle(mFrame_mc, 
							imgX - fw,
							imgY - fw,
							imgX + imgW + fw,
							imgY + imgH + fw,
							fw);							
		mFrame_mc.endFill();
		
										
		if (!Options.captionUnderThumbs){
			mStageManager.setCaptionPosition(imgX - fw, imgY + imgH + fw,imgW); 
		}
	}
	
	public function setPosn(x:Number,y:Number):Void{
		mClip_mc._x = x;
		mClip_mc._y = y;
	}
	
	public function showImageProgress(percent:Number){				
		mImgLoadStatus_mc.mcBar._width = Math.floor(percent/100*mImgLoadStatus_mc.mcBkgnd._width);	
		mImgLoadStatus_mc._visible = true;		
	}
		
	public function get imageWidth():Number{		
		return mLoader_mc._width;		
	}
	
	public function get imageHeight():Number{		
		return mLoader_mc._height;		
	}
	
	public function get imageX():Number{		
		return mLoader_mc._x;		
	}
	
	public function get imageY():Number{		
		return mLoader_mc._y;		
	}
	
	public function get imageLoadError():Boolean{
		return mImageLoadError;	
	}
}