//****************************************************************************
//Copyright (C) 2008 Airtight Interactive. All Rights Reserved.
//Usage of this code is subject to the Terms of Use accompanying
//this product.
//****************************************************************************

import mx.transitions.Tween;
import mx.utils.Delegate;

import com.airtightinteractive.apps.viewers.simpleViewer.Image;
import com.airtightinteractive.apps.viewers.simpleViewer.Options;
import com.airtightinteractive.apps.viewers.simpleViewer.RolloverButton;
import com.airtightinteractive.apps.viewers.simpleViewer.StageManager;
import com.airtightinteractive.apps.viewers.simpleViewer.ThumbArea;
import com.airtightinteractive.apps.viewers.simpleViewer.XMLManager;
import com.airtightinteractive.utils.ArrayUtil;

/**
 * ImageArea is the holder that containd the large images.
 * Handles image arrows navigation, showing image error icon, resizing,
 * loading images in sequence.
 * 
 * @author Felix Turner
 */

class com.airtightinteractive.apps.viewers.simpleViewer.ImageArea {
		
	private static var instance : ImageArea;
	
	private var mStageManager:StageManager;
	private var mThumbArea: ThumbArea;		
	private var mXMLManager:XMLManager;
	
	private var mImages:Array;//All images that have/are being loaded
	private var mImageLoadQueue:Array;//list of image ids in order to be loaded	
	private var mCurrentImageIndex:Number;	
	private var mWidth:Number; //width of image area including frame
	private var mHeight:Number; //heigth of image area including frame	
	private var mNextBtn:RolloverButton;
	private var mBackBtn:RolloverButton;		
	private var mPlayBtn:RolloverButton;		
	private var mBtnFadeTween:Tween;
	
	private var mClip_mc:MovieClip;
	private var mImageHolder_mc:MovieClip;
	private var mButtonHolder_mc:MovieClip;
	private var mError_mc:MovieClip;
	
	private var mPlaying:Boolean;
	private var mPlay_int:Number;	
		
	/**
	 * @return singleton instance of ThumbArea
	 */
	public static function getInstance() : ImageArea {
		if (instance == null)
			instance = new ImageArea();
		return instance;		
	}
	
	private function ImageArea() {
		
	}
	
	/**
	 * Initialize Image Area
	 */
	 function init(target:MovieClip){
		
		mClip_mc = target.createEmptyMovieClip("imageArea",target.getNextHighestDepth());	
				
		mXMLManager = XMLManager.getInstance();
		mStageManager = StageManager.getInstance();
		mThumbArea = ThumbArea.getInstance();	
		mImages = [];	
		mImageLoadQueue = [];
		mCurrentImageIndex = -1;	
		
		mImageHolder_mc = mClip_mc.createEmptyMovieClip("mcFrame",mClip_mc.getNextHighestDepth());			
		mButtonHolder_mc = mClip_mc.createEmptyMovieClip("mcBtn",mClip_mc.getNextHighestDepth());
		
   		//init image navigation buttons   		
   		if (Options.enableImageButtons){
   			//next
			mButtonHolder_mc.attachMovie("ImageNextButton","mcNextBtn",mButtonHolder_mc.getNextHighestDepth());   			
			mNextBtn = new RolloverButton(mButtonHolder_mc.mcNextBtn);    		
	   		mNextBtn.doAction = Delegate.create(this, onClickNext);  
	   		mNextBtn.setRGB(mXMLManager.imageFrameColor);	
			//back
			mButtonHolder_mc.attachMovie("ImageBackButton","mcBackBtn",mButtonHolder_mc.getNextHighestDepth());	
			mBackBtn = new RolloverButton(mButtonHolder_mc.mcBackBtn);    		
	   		mBackBtn.doAction = Delegate.create(this, onClickBack);
	   		mBackBtn.setRGB(mXMLManager.imageFrameColor);
		   				
			mBtnFadeTween = new Tween(mButtonHolder_mc,"_alpha",null,0,0,1,false);	
			mButtonHolder_mc._visible = false;	
			
			mBackBtn.hideOnRollOut = Options.hideImageButtonsOnRollOut;
			mNextBtn.hideOnRollOut = Options.hideImageButtonsOnRollOut;						
   		}
		
		if (Options.enablePlayButton){
			//play
			mButtonHolder_mc.attachMovie("PlayButton","mcPlayBtn",mButtonHolder_mc.getNextHighestDepth());	
			mPlayBtn = new RolloverButton(mButtonHolder_mc.mcPlayBtn);    		
	   		mPlayBtn.doAction = Delegate.create(this, onClickPlay);
	   		mPlayBtn.setRGB(mXMLManager.imageFrameColor);	
			mPlayBtn.hideOnRollOut = Options.hideImageButtonsOnRollOut;	
		}
   		
   		//init error icon
   		mError_mc = mClip_mc.attachMovie("ErrorIcon","error",mClip_mc.getNextHighestDepth());
   		mError_mc._visible = false;
   		var col = new Color(mError_mc);
		col.setRGB(mXMLManager.imageFrameColor);
						
		Key.addListener(this);		
		setPlaying(Options.playAtStart);
	}	
	
	public function createImages():Void{
		for (var i : Number = 0; i <= mXMLManager.imageCount-1; i++) {			
			mImages[i] = new Image(mImageHolder_mc, i, mXMLManager.imagePath + mXMLManager.imageFileNames[i],this);
			mImageLoadQueue[i] = i;					
		}		
		mImageLoadQueue.reverse();	
		
		//load 1st image		
		loadNextImage();
		
		//jump directly to an image if set in flashvars
		if (_root.firstImageIndex != undefined){
			mThumbArea.selectedThumbIndex = _root.firstImageIndex;
		}	
		
			
	}
	
	public function loadNextImage(){		
		var nextImgId = mImageLoadQueue.pop();		
		mImages[nextImgId].loadImage(true);		
	}
	
	/**
	 * Thumbnail has been clicked out of sequence, load this image early
	 */
	public function loadImageOutOfSequence(imageIndex:Number){	
		//remove from queue
		var i = ArrayUtil.indexOf(mImageLoadQueue,imageIndex);
		if (i > -1){
			mImageLoadQueue.splice(i,1);
			mImages[imageIndex].loadImage(false);
		}					
	}
		
	public function setSize(w:Number,h:Number):Void{	
			
		var fw = mXMLManager.frameWidth;
		mClip_mc._visible = (w > fw*2 && h > fw*2);	
					
		mWidth = Math.round(w);
		mHeight = Math.round(h);			
		//resize current image
		mImages[mCurrentImageIndex].resizeImage();						 	
		mError_mc._x = mWidth/2;
		mError_mc._y = mHeight/2;			
		resizeImageNavButtons();
				
		//image nav hit areas remain same size as imageArea
		mBackBtn.setHitPosn(0,0);
		mBackBtn.setHitSize(mWidth/3+fw,mHeight);		
		mNextBtn.setHitPosn(mWidth*2/3-fw,0);
		mNextBtn.setHitSize(mWidth/3+ fw,mHeight);
		mPlayBtn.setHitPosn(mWidth/3+fw,0);
		mPlayBtn.setHitSize(mWidth/3,mHeight);
	}
	
	public function setPosn(x:Number,y:Number):Void{
		x = Math.round(x);
		y = Math.round(y);		
		mClip_mc._x = x;
		mClip_mc._y = y;			
	}
	
	public function showImage(imgIndex:Number):Void{	
				
		//trans out old img	
		if 	(imgIndex != mCurrentImageIndex) mImages[mCurrentImageIndex].hideImage();		
		mCurrentImageIndex = imgIndex;	
					
		if (mImages[mCurrentImageIndex].imageLoadError){
			mError_mc._visible = true;	
			mImages[mCurrentImageIndex].hideImage();			
		}else{
			mError_mc._visible = false;					 
			mImages[mCurrentImageIndex].showImage();					
		}	
			
		//hide image nav buttons
		mBackBtn.visible = mCurrentImageIndex > 0;
		mNextBtn.visible = Options.enableLooping || mCurrentImageIndex < mXMLManager.imageCount-1;		
		mPlayBtn.visible = Options.enableLooping || mCurrentImageIndex < mXMLManager.imageCount-1;		
		//show caption
		mStageManager.showCaption(mCurrentImageIndex);	
						
		//setPlaying(false); //manual navigation disables autoplay
	}
		
	public function fadeInButtons(){
		resizeImageNavButtons();
		mBtnFadeTween.continueTo(0,1);
		mBtnFadeTween.fforward();
		mBtnFadeTween.continueTo(100,Options.imageFadeLength);		
	}
	
	public function fadeOutButtons(){
		resizeImageNavButtons();
		mBtnFadeTween.continueTo(100,1);
		mBtnFadeTween.fforward();
		mBtnFadeTween.continueTo(0,Options.imageFadeLength);				
	}
	
	private function resizeImageNavButtons(){			
		
		//show image nav buttons if image is loaded and not an error	
		if (mImages[mCurrentImageIndex].loaded && !mImages[mCurrentImageIndex].imageLoadError){
			
			mButtonHolder_mc._visible = true;				
			var w = mImages[mCurrentImageIndex].imageWidth;
			var h = mImages[mCurrentImageIndex].imageHeight;		
			var imgx = mImages[mCurrentImageIndex].imageX;
			var imgy = mImages[mCurrentImageIndex].imageY;		
			var fw = mXMLManager.frameWidth;			
			var btnW = mBackBtn.btnWidth;								
			mBackBtn.setBtnPosn(imgx - mBackBtn.clickShift,imgy + h/2  - btnW/2);		
			mNextBtn.setBtnPosn(imgx + w - btnW,imgy + h/2  - btnW/2);	
			mPlayBtn.setBtnPosn(imgx + w/2 - mPlayBtn.btnWidth/2,imgy + h - mPlayBtn.btnHeight);							
		}else{
			mButtonHolder_mc._visible = false;				
		}	
	}
	
	private  function onClickNext():Void{	
		mThumbArea.selectedThumbIndex++;	
	}
	
	private function onClickBack():Void{		
		mThumbArea.selectedThumbIndex--;		
	}
	
	private  function onClickPlay():Void{	
		setPlaying(!mPlaying);
		if 	(mPlaying){	
			mThumbArea.selectedThumbIndex++;
			setPlaying(true);
		}		
	}
	
	public function setPlaying(play:Boolean){
				
		mPlaying = play;
		clearInterval(mPlay_int);		
		mPlayBtn.buttonFrame = mPlaying ? 1: 2;
		if (mPlaying){			
			mPlay_int = setInterval(Delegate.create(this,doStep),Options.displayTime*1000);			
		}	
	}
	
	private function doStep(){		
		
		if (mThumbArea.selectedThumbIndex >= mXMLManager.imageCount-1 && !Options.enableLooping){
			setPlaying(false);
		}else{
			mThumbArea.selectedThumbIndex++;
			setPlaying(true);
		}
	}
	
	public function openImageLink(){
		getURL(mXMLManager.imagePath + mXMLManager.imageFileNames[mCurrentImageIndex],"_blank");	
	}
	
	private function onKeyDown() {			
		if(Key.isDown(Key.SPACE)) {	
			onClickPlay();										
		}
	}

	public function get currentImageIndex():Number{
		return mCurrentImageIndex;	
	}	
	
	public function get width():Number{		
		return mWidth;		
	}
	
	public function get height():Number{		
		return mHeight;		
	}
	
	private function openURL(){
		mStageManager.openURL();
	}
	
	public function get xpos():Number{
	   return mClip_mc._x;
	}
	   
	public function get ypos():Number{
	   return mClip_mc._y;
	}
			
}