//****************************************************************************
//Copyright (C) 2008 Airtight Interactive. All Rights Reserved.
//Usage of this code is subject to the Terms of Use accompanying
//this product.
//****************************************************************************

/**
 * SimpleViewer options. Change values in this file to modify the behaviour of SimpleViewer. 
 * For instructions check the documentation:
 * http://www.airtightinteractive.com/simpleviewer/pro/documentation.html#asoptions
 * 
 * @author Felix Turner
 */
class com.airtightinteractive.apps.viewers.simpleViewer.Options {
		
	function Options(){};
	
	//General
	static var fixedLayout:Boolean = false; //Whether to used fixed layout of stage elements, or dynamic resizing based on browser window size  		
	static var preloadImages:Boolean = true; //Whether to automatically preload images in the background 
	static var useSmoothing:Boolean = true; //Whether to smooth images
	static var enableMouseWheel:Boolean = false; //Whether to accept mouse wheel input to navigate images
	static var enableLooping:Boolean = false; //Whether navigating beyond last image goes back to first image	
	static var soundPath:String = null; //Relative or absolute path to optional music mp3 file.
		
	//Main Image	
	static var enableImageDownScaling:Boolean = true; //Whether images are resized smaller to fit stage (not used if fixedLayout == true)
	static var enableImageButtons:Boolean = true; //Whether to use image navigation buttons (arrow buttons that overlay the main image)
	static var hideImageButtonsOnRollOut:Boolean = true; //Whether to hide the image navigation buttons when the mouse is not rolled over them
	static var imageFadeLength:Number = 5; //Image fade tween length (frames)
	static var imageVAlign:String = "center"; //Vertical placment of the image within the square image area. Can be "center", "top" or "bottom".
	static var imageHAlign:String = "center"; //Horizontal placement of the image within the square image area. Can be "center", "left" or "right".
	
	//Thumbnails
	static var thumbnailWidth:Number = 65; //pixels
	static var thumbnailHeight:Number = 65; //pixels	
	static var thumbnailPadding:Number = 14; //space between thumbs (pixels)
	static var showThumbnailVisited:Boolean = true; //Whether to show visited dogear icon
	static var showThumbnailLoadBar:Boolean = true; //Whether to show load bar on image load
	static var thumbnailFrameWidth:Number = 2; //pixels
	static var thumbnailRollOverFrameWidth:Number = 5;	 //pixels
	static var thumbnailSelectedFrameOffset:Number = 3; //pixels that thumb frame is offset  when thumb is selected
	static var thumbnailLoadBarHeight:Number = 10; //pixels
	static var thumbnailClickShift:Number = 2;	 //distance thumbnail moves on click (pixels)
	static var thumbnailLoadingAlpha:Number = 60; //alpha of thumbnail while image loads (percent)
	static var thumbnailMotionLength:Number = 20; //Thumb paging slide animation tween time (frames)
								
	//Captions
	static var showCaptions:Boolean = true; //Whether to show Image Captions	
	static var showTitle:Boolean = true; //Whether to show gallery title
	static var showCaptionOnRollover:Boolean = false; //Whether to show the image caption on thumbnail rollover
	static var captionUnderThumbs:Boolean = true; //Whether to show the captions under the thumbnails or the main image
	static var captionFontSize:Number = 13;
	static var titleFontSize:Number = 16;	
	static var captionPadding:Number = 14; //space between image and caption. Used if 'captionUnderThumbs' is false
	
	//Autoplay
	static var enablePlayButton:Boolean = false; //Whether to show play button for autoplay mode.
	static var playAtStart:Boolean = true; //Whether to automatically start playing.
	static var displayTime:Number = 8; //Number of seconds each image will display in autoplay mode.	
	
	//Back Button
	static var showBackButton:Boolean = false; //Whether to show optional back button
	static var backButtonText:String = "<b>&lt; <u>Back</b></u>"; //Display text for optional back button
							
	//Colors
	//If these are not set, color are loaded from XML options
	//Color values are hex-numbers like this: 0xFF00FF
	static var imageFrameColor:Number = null; //color of main image frame and image nav buttons (overrides XML 'frameColor' value)
	static var thumbnailFrameColor:Number = null; //color of thumbnail frames	(overrides XML 'frameColor' value)
	static var pagingArrowsColor:Number = null; //color of thumbnail paging arrows (overrides XML 'frameColor' value)
	static var captionTextColor:Number = null; //color of captions text (overrides XML 'textColor' value)
	static var titleTextColor:Number = null; //color of title text (overrides XML 'textColor' value)
	
	//Fixed Layout Positions
	//These options are only used if 'fixedLayout' is set to true
	//ThumbArea width and Height are determined by columns, rows, thumbnailWidth, thumbnailHeight and thumbnailPadding
	static var imageAreaX:Number = 10; //X posn of Image Area
	static var imageAreaY:Number = 10; //Y posn of Image Area
	static var imageAreaWidth:Number = 480; //width of Image Area
	static var imageAreaHeight:Number = 480; //height of Image Area	
	static var thumbAreaX:Number = 500; //X posn of Thumb Area
	static var thumbAreaY:Number = 10; //Y posn of Thumb Area 		
	static var backThumbArrowX:Number = 0; //X posn of Back Thumb Arrow (relative to top left of Thumb Area)
	static var backThumbArrowY:Number = 350; //Y posn of Back Thumb Arrow (relative to top left of Thumb Area)
	static var nextThumbArrowX:Number = 200; //X posn of Next Thumb Arrow (relative to top left of Thumb Area)
	static var nextThumbArrowY:Number = 350; //Y posn of Next Thumb Arrow (relative to top left of Thumb Area)	
	static var titleX:Number = 10; //X posn of Title text
	static var titleY:Number = 500; //Y posn of Title text
	static var titleWidth:Number = 480; //width of Title text
	static var titleHeight:Number = 20; //height of Title text		
	static var captionX:Number = 10; //X posn of Title text
	static var captionY:Number = 520; ///Y posn of Title text
	static var captionWidth:Number = 480; //width of Title text
	static var captionHeight:Number = 50; //height of Title text	
	static var downLoadLinkX:Number = 600; //X posn of download link
	static var downloadLinkY:Number = 600; //Y posn of download link
	static var backButtonX:Number = 20; //X posn of optional back button
	static var backButtonY:Number = 20; //Y posn of optional back button
	
	//XML Options Overrides
	//Setting these values will replace the values loaded from the XML doc
	static var maxImageWidth:Number = null;
	static var maxImageHeight:Number = null;
	static var frameWidth:Number = null;
	static var stagePadding:Number = null;
	static var navPadding:Number = null;
	static var thumbnailColumns:Number = null;
	static var thumbnailRows:Number = null;
	static var navPosition:String = null;  
	static var vAlign:String = null;
	static var hAlign:String = null;	
	static var title:String = null;
	static var enableRightClickOpen:Boolean = null; 
	static var backgroundImagePath:String = null; 	
	static var imagePath:String = null;  
	static var thumbPath:String = null;	
}