//****************************************************************************
//Copyright (C) 2008 Airtight Interactive. All Rights Reserved.
//Usage of this code is subject to the Terms of Use accompanying
//this product.
//****************************************************************************

import flash.external.ExternalInterface;

import mx.utils.Delegate;

import com.airtightinteractive.apps.viewers.simpleViewer.ImageArea;
import com.airtightinteractive.apps.viewers.simpleViewer.Options;
import com.airtightinteractive.apps.viewers.simpleViewer.Thumb;
import com.airtightinteractive.apps.viewers.simpleViewer.ThumbArea;
import com.airtightinteractive.apps.viewers.simpleViewer.XMLManager;

/**
 * Main class. Handles stage layout, title and caption text.
 * @author Felix Turner
 */

class com.airtightinteractive.apps.viewers.simpleViewer.StageManager {
	
	private static var instance : StageManager;
	private static var IS_PRO:Boolean = true;
	private static var VERSION:String = "1.9";
	private static var sResizeTime:Number = 100;
	//DEBUT	MODIF_SK
	//private static var linkURL:String = "http://www.airtightinteractive.com/simpleviewer/"; 
	private static var linkURL:String = "http://www.studiok-web.com/"; //URL used when download link is clicked
	//FIN MODIF
	private var langAbout:String = "About";
	private var langOpenImage:String = "Open Image in New Window";		
	
	private var mXMLManager : XMLManager;	
	private var mThumbArea:ThumbArea;
	private var mImageArea:ImageArea;	
	private var mResize_int:Number;
	private var mBkgndLoaded:Boolean;	
	private var mClip_mc: MovieClip;
	private var mDownloadLink_mc: MovieClip;
	private var mCaption_mc: MovieClip;
	private var mTitle_mc: MovieClip;
	private var mBkgnd_mc:MovieClip;
	private var mBackBtn_mc:MovieClip;
	private var musicSnd:Sound;
		
	/**
	 * @return singleton instance of StageManager
	 */
	public static function getInstance() : StageManager {
		if (instance == null)
			instance = new StageManager();
		return instance;
	}
	
	private function StageManager() {
		
	}
	
	/**
	 * Initialize app
	 */
	public function init(target:MovieClip):Void{
	
		mClip_mc = target;
		
		//get localization language
		if (_root.langAbout != undefined){
			langAbout = _root.langAbout;
		}
		if (_root.langOpenImage != undefined){
			langOpenImage = _root.langOpenImage;
		}
		
		//create stage elements
		mThumbArea = ThumbArea.getInstance();
		mImageArea = ImageArea.getInstance();
		mXMLManager = XMLManager.getInstance();							
		mXMLManager.xmlData =  _global.gImageData_xml;
								
		//init right click menu		
		var stage_cm:ContextMenu = new ContextMenu ();
		stage_cm.hideBuiltInItems();
		if (mXMLManager.enableRightClickOpen){
			stage_cm.customItems.push(new ContextMenuItem(langOpenImage+"...", Delegate.create(this,openImageLink)));		
		}
		if (!IS_PRO){
			stage_cm.customItems.push(new ContextMenuItem(langAbout + " SimpleViewer "+ VERSION+"...", Delegate.create(this,openURL)));									
		}		
		//DEBUT	MODIF_SK
		stage_cm.customItems.push(new ContextMenuItem("Studio K - création de sites Internet", Delegate.create(this,openURL)));	
		//FIN MODIF
		mClip_mc.menu = stage_cm;
				
		mBkgndLoaded = false;			
	}	
		
	/**
	 * Initialize elements on the stage 
	 */
	 function onXMLLoaded(){
		
		//load optional background image
		if (mXMLManager.backgroundImagePath != null){
			mBkgnd_mc = mClip_mc.createEmptyMovieClip("mcBkgnd",mClip_mc.getNextHighestDepth());
			var loader_mcl:MovieClipLoader = new MovieClipLoader();
			loader_mcl.addListener(this);		
			loader_mcl.loadClip(mXMLManager.backgroundImagePath,mBkgnd_mc);						
		}	
									
		mImageArea.init(mClip_mc);
		mThumbArea.init(mClip_mc);			
		
		//init text	
		if (Options.showCaptions && mThumbArea.pageSize > 0){			
			mCaption_mc = mClip_mc.attachMovie("Caption","mcCaption",mClip_mc.getNextHighestDepth());
			if (!Options.fixedLayout) mCaption_mc.txtText.autoSize = true;
			mCaption_mc.txtText.wordWrap = true;
			if (!Options.captionUnderThumbs) mCaption_mc._visible = false;			
		}
		
		if (Options.showTitle && mThumbArea.pageSize > 0){			
			mTitle_mc = mClip_mc.attachMovie("Title","mcTitle",mClip_mc.getNextHighestDepth());		
			if (!Options.fixedLayout) mTitle_mc.txtText.autoSize = true;
			mTitle_mc.txtText.wordWrap = true;		
			mTitle_mc.txtText.htmlText = "<font color=\"#" + ARGBHexToString(mXMLManager.titleTextColor) + "\" size=\"" + Options.titleFontSize + "\">" + mXMLManager.title + "</font>";			

		}
		
		//init download link   		
   		if (!IS_PRO){
			mDownloadLink_mc = mClip_mc.attachMovie("DownloadLink","mcDownloadLink",mClip_mc.getNextHighestDepth());						
			mDownloadLink_mc.onRelease = Delegate.create(this,openURL);																
		}
		
		//init optional 'back' button
		if (Options.showBackButton){	
			mBackBtn_mc = mClip_mc.attachMovie("Caption","mcBackBtn",mClip_mc.getNextHighestDepth());
			mBackBtn_mc.txtText.autoSize = true;
			mBackBtn_mc.txtText.wordWrap = false;
			mBackBtn_mc.txtText.htmlText = "<font color=\"#" + ARGBHexToString(mXMLManager.captionTextColor) + "\" size=\"" + Options.captionFontSize + "\">" + Options.backButtonText + "</font>";			
			mBackBtn_mc.onRelease = Delegate.create(this,goBack);	
		}
		
   		//init resizing		
		if (!Options.fixedLayout) Stage.addListener(this);	
		doLayout();
		
		//start loading thumbs	
		mImageArea.createImages();
		mThumbArea.createThumbs();	
				
		//load optional music
		if (Options.soundPath != null){
			musicSnd = new Sound();
			musicSnd.loadSound(Options.soundPath,true);
			musicSnd.onSoundComplete = function() {
				this.start(0);
			}
		}
	}
	
	/**
	 * Layout elements on the stage. Called when browser window is resized
	 * 
	 */
	function doLayout(){
		
		clearInterval(mResize_int);
		mResize_int = null;
		
		if (Options.fixedLayout){
			
			//Fixed Layout. Set stage element positions based on Options. 	
			mThumbArea.setPosn(Options.thumbAreaX,Options.thumbAreaY);			
			mImageArea.setPosn(Options.imageAreaX,Options.imageAreaY);
			mImageArea.setSize(Options.imageAreaWidth,Options.imageAreaHeight);
			
			mCaption_mc._x = Options.captionX;
			mCaption_mc._y = Options.captionY;			
			mCaption_mc.txtText._width = Options.captionWidth;
			mCaption_mc.txtText._height = Options.captionHeight;
			
			mTitle_mc._x = Options.titleX;
			mTitle_mc._y = Options.titleY;
			mTitle_mc.txtText._width = Options.titleWidth;
			mTitle_mc.txtText._height = Options.titleHeight;
			
			mDownloadLink_mc._x = Options.downLoadLinkX;
			mDownloadLink_mc._y = Options.downloadLinkY;
			
			mBackBtn_mc._x = Options.backButtonX;
			mBackBtn_mc._y = Options.backButtonY;
									
		}else{
			
			//Dynamic Layout - elements resize to fit stage			
			var w:Number = Stage.width;
			var h:Number = Stage.height;
			
			//use fixed stage dimensions if defined in container FLA
			if (_global.SVStageWidth != undefined) w = _global.SVStageWidth;
			if (_global.SVStageHeight != undefined) h = _global.SVStageHeight;		
				
			//calculate positions of imageArea and ThumbArea based on XML options					
			var imgH:Number; 
			var imgW:Number;
			var imgX:Number;
			var imgY:Number;			
			var thumbX:Number;
			var thumbY:Number;
			var imgPlusThumbsW:Number;
			var imgPlusThumbsH:Number;
			var imgPlusThumbsX:Number;
			var imgPlusThumbsY:Number;	
			
			var sp:Number = mXMLManager.stagePadding;
			var tp:Number = mXMLManager.navPadding;
			var fw:Number = mXMLManager.frameWidth;			
			var thumbH:Number = mThumbArea.height + mThumbArea.navHeight;	
			var thumbW:Number = mThumbArea.width + mThumbArea.navWidth
						
			var verticalNav:Boolean = mXMLManager.navPosition == "top" || mXMLManager.navPosition == "bottom";								
			
			//calc img h + w
			if (Options.enableImageDownScaling){												
				var vSpacing:Number = 2*sp;
				var hSpacing:Number = 2*sp;													
				if (verticalNav){ 
					vSpacing += tp + thumbH;										
				}else{	
					hSpacing += tp + thumbW;									
				}				
				imgH =  Math.min(h-  vSpacing,mXMLManager.maxImageHeight + fw*2);					
				imgW = Math.min(w-  hSpacing ,mXMLManager.maxImageWidth + fw*2);										
				
			}else{
				imgW = mXMLManager.maxImageWidth;
				imgH = mXMLManager.maxImageHeight;				
			} 
			
			if (verticalNav){ 
				imgPlusThumbsW = imgW;
				imgPlusThumbsH = imgH + tp + thumbH;
			}else{
				imgPlusThumbsW = imgW + tp + thumbW;
				imgPlusThumbsH = imgH;
			}			
			
			//calc Y posn
			switch (mXMLManager.vAlign) { 			
				case "top":
					imgPlusThumbsY = sp;
					break;
				
				case "bottom":
					imgPlusThumbsY = h - imgPlusThumbsH - sp;
					break;
					
				default : //center					
					imgPlusThumbsY = (h - imgPlusThumbsH)/2;													
					break;
			}
			
			//calc X posn
			switch (mXMLManager.hAlign) { 
			
				case "left":
					imgPlusThumbsX = sp;
					break;
				
				case "right":
					imgPlusThumbsX = w - imgPlusThumbsW - sp;
					break;
					
				default : //center
					imgPlusThumbsX = (w - imgPlusThumbsW)/2;				
					break;
			}
			
			//calc thumb + img posns
			switch (mXMLManager.navPosition) { 
			 	
				case "right" : 
					imgX = imgPlusThumbsX;
					imgY = imgPlusThumbsY;
					thumbX = imgX + imgW + tp;
					thumbY = imgPlusThumbsY + imgH/2 - thumbH/2; //vcenter thumbs to img
					break; 
					
				case "top" : 				
					thumbY = imgPlusThumbsY;
					thumbX = imgPlusThumbsX + imgW/2 - thumbW/2; //hcenter thumbs to img
					imgX = imgPlusThumbsX;
					imgY = thumbY + thumbH + tp;					
					break; 
					
				case "bottom" : 
					imgX = imgPlusThumbsX;
					imgY = imgPlusThumbsY;
					thumbX = imgPlusThumbsX + imgW/2 - thumbW/2; //hcenter thumbs to img
					thumbY = imgY + imgH + tp
					break;
					
				default :	//left
					thumbX = imgPlusThumbsX;
					thumbY = imgPlusThumbsY + imgH/2 - thumbH/2; //vcenter thumbs to img
					imgX = thumbX + thumbW + tp;
					imgY = imgPlusThumbsY 
					break;
			}
									
			//set posns
			mImageArea.setPosn(imgX,imgY);	
			mImageArea.setSize(imgW,imgH);															
			mThumbArea.setPosn(thumbX,thumbY);
			
			//caption
			if (Options.captionUnderThumbs){
				mCaption_mc.txtText._width = thumbW;
				mCaption_mc._x = thumbX;			
				mCaption_mc._y = Math.floor(thumbY + thumbH + Thumb.padding);
			}
			
			//title
			mTitle_mc.txtText._width = thumbW;
			mTitle_mc._x = thumbX;			
			mTitle_mc._y = Math.floor(thumbY -  mTitle_mc.txtText.textHeight - Thumb.padding);
			
			//download link
			mDownloadLink_mc._x = Math.floor (w  - mDownloadLink_mc._width - 10);
			mDownloadLink_mc._y = Math.floor (h  - mDownloadLink_mc._height - 10);
			
			//back button
			mBackBtn_mc._x = 10;
			mBackBtn_mc._y = 10;
												
			resizeBkgnd();					
		}				
	}	
	
	public function showCaption(index:Number){
		var cap = mXMLManager.imageCaptions[index];
		if (cap == undefined) cap = ""; 		
		mCaption_mc.txtText.htmlText = "<font color=\"#" + ARGBHexToString(mXMLManager.captionTextColor) + "\" size=\"" + Options.captionFontSize + "\">" + cap + "</font>";			
	}
	
	public function openURL(){
		getURL(linkURL,"_blank");			
	}
	
	public function goBack(){		
		if (ExternalInterface.available) {
			ExternalInterface.call("history.back");
		}
	}
		
	/**
	 * Call doLayout when the stage is resized. Only resize every 100 ms to avoid choking
	 */	
	private function onResize(){				
		if (mResize_int == null){
			mResize_int = setInterval(Delegate.create(this,doLayout),sResizeTime);
		}	
	}
	
	/*
	 * Bkgnd image has loaded.	 
	 */
	private function onLoadInit(){
		mBkgnd_mc.forceSmoothing = Options.useSmoothing;
		mBkgndLoaded = true;
		resizeBkgnd();
	}
	
	private function resizeBkgnd(){
		//resize optional bkgnd image to fit
		if (mXMLManager.backgroundImagePath != null && mBkgndLoaded && !Options.fixedLayout){
			mBkgnd_mc._width = Stage.width;
			mBkgnd_mc._height = Stage.height;
		}
	}	
	
	private function openImageLink(){
		mImageArea.openImageLink();		
	}
	
	function ARGBHexToString(p_hex:Number):String {	
		return (((p_hex>> 24) != 0) ? Number((p_hex>>> 24) & 0xFF).toString(16) : "")+Number(p_hex & 0xFFFFFF).toString(16);
	}
	
	public function setCaptionPosition(x,y,w){
		if (isNaN(x) || Options.fixedLayout) return;
		mCaption_mc.txtText._width = w;
		mCaption_mc._x = mImageArea.xpos + x;
		mCaption_mc._y = mImageArea.ypos + y + Options.captionPadding;
		mCaption_mc._visible = true;
	}
	
	public function setCaptionVisible(show:Boolean){
		mCaption_mc._visible = show;
	}
			
}