//****************************************************************************
//Copyright (C) 2008 Airtight Interactive. All Rights Reserved.
//Usage of this code is subject to the Terms of Use accompanying
//this product.
//****************************************************************************

import com.airtightinteractive.apps.viewers.simpleViewer.StageManager;
import com.airtightinteractive.apps.viewers.simpleViewer.ThumbArea;
import com.airtightinteractive.apps.viewers.simpleViewer.Options;

/**
 * Handles xml parsing. Sets app settings based on XML attributes.
 * If XML values are missing, use default settings defined here.
 * If override values defined in Options class, override values.
 * @author Felix Turner
 */

class com.airtightinteractive.apps.viewers.simpleViewer.XMLManager {
	
	private static var instance : XMLManager;
	private var mImageData_xml:XML;
	private var mStageManager:StageManager;	
	private var mThumbArea: ThumbArea;
				
	//loaded XML options
	public var maxImageHeight:Number = 640;
	public var maxImageWidth:Number = 640;
	public var stagePadding:Number = 40;
	public var navPadding:Number = 40;
	public var imageFrameColor:Number = 0xFFFFFF;
	public var thumbnailFrameColor:Number = 0xFFFFFF;
	public var pagingArrowsColor:Number = 0xFFFFFF;	
	public var captionTextColor:Number = 0xFFFFFF; 
	public var titleTextColor:Number = 0xFFFFFF;
	public var frameWidth:Number = 20;	
	public var thumbnailColumns:Number = 3;
	public var thumbnailRows:Number = 3;
	public var navPosition:String = "left";
	public var hAlign:String = "center";
	public var vAlign:String = "center";
	public var imagePath:String = "images/";    
	public var thumbPath:String = "thumbs/";  	
	public var title:String = "";  
	public var enableRightClickOpen:Boolean = false;
	public var backgroundImagePath:String = null;
	
	//loaded XML images data
	public var imageCount:Number;  
	public var imageFileNames:Array;
	public var imageCaptions:Array;	
						
	/**
	 * @return singleton instance of XMLManager
	 */
	public static function getInstance() : XMLManager {
		if (instance == null)
			instance = new XMLManager();
		return instance;
	}
		
	private function XMLManager() {			
		mStageManager = StageManager.getInstance();		
		mThumbArea = ThumbArea.getInstance();			
	} 		
	
	/**
	 * Parse loaded gallery data XML
	 */
	public function set xmlData(xmld:XML){
				
		mImageData_xml = xmld;
			
		imageCount = 0; 
		imageFileNames = [];
		imageCaptions = [];	
				
		var xmlRoot = mImageData_xml.firstChild;
		
		if (xmlRoot == null) return; //XML is malformed
		
		//Get values from XML, if values missing keep default values				
		maxImageWidth = setIfValidNumber(xmlRoot.attributes.maxImageWidth,maxImageWidth);
		maxImageHeight = setIfValidNumber(xmlRoot.attributes.maxImageHeight,maxImageHeight);
		
		//make backward compatible with maxImageDimension
		maxImageWidth = setIfValidNumber(xmlRoot.attributes.maxImageDimension,maxImageWidth);
		maxImageHeight = setIfValidNumber(xmlRoot.attributes.maxImageDimension,maxImageHeight);
				
		frameWidth = setIfValidNumber(xmlRoot.attributes.frameWidth,frameWidth);
		stagePadding = setIfValidNumber(xmlRoot.attributes.stagePadding,stagePadding);
		navPadding = setIfValidNumber(xmlRoot.attributes.navPadding,navPadding);
		thumbnailColumns = setIfValidNumber(xmlRoot.attributes.thumbnailColumns,thumbnailColumns);
		thumbnailRows = setIfValidNumber(xmlRoot.attributes.thumbnailRows,thumbnailRows);
		imageFrameColor = setIfValidNumber(xmlRoot.attributes.frameColor,imageFrameColor);
		thumbnailFrameColor = setIfValidNumber(xmlRoot.attributes.frameColor,thumbnailFrameColor);
		pagingArrowsColor = setIfValidNumber(xmlRoot.attributes.frameColor,pagingArrowsColor);
		captionTextColor = setIfValidNumber(xmlRoot.attributes.textColor,captionTextColor);
		titleTextColor = setIfValidNumber(xmlRoot.attributes.textColor,titleTextColor);
				
		if (isValidPath(xmlRoot.attributes.imagePath)) imagePath = xmlRoot.attributes.imagePath;				
		if (isValidPath(xmlRoot.attributes.thumbPath)) thumbPath = xmlRoot.attributes.thumbPath;
		if (isValidPath(xmlRoot.attributes.backgroundImagePath)) backgroundImagePath = xmlRoot.attributes.backgroundImagePath;				
		if (xmlRoot.attributes.title != undefined) title = xmlRoot.attributes.title;	
		
		navPosition = xmlRoot.attributes.navPosition;
		hAlign = xmlRoot.attributes.hAlign;
		vAlign = xmlRoot.attributes.vAlign;						
		
		if (xmlRoot.attributes.enableRightClickOpen == "true") {
			enableRightClickOpen = true;
		}else if (xmlRoot.attributes.enableRightClickOpen == "false") {
			enableRightClickOpen = false;
		}
					
		//If options are set in Options class, override values
		if (Options.maxImageWidth != null) maxImageWidth = Options.maxImageWidth;
		if (Options.maxImageHeight != null) maxImageHeight = Options.maxImageHeight;
		if (Options.imageFrameColor != null) imageFrameColor = Options.imageFrameColor;
		if (Options.thumbnailFrameColor != null) thumbnailFrameColor = Options.thumbnailFrameColor;
		if (Options.pagingArrowsColor != null) pagingArrowsColor = Options.pagingArrowsColor;
		if (Options.captionTextColor != null) captionTextColor = Options.captionTextColor;
		if (Options.titleTextColor != null) titleTextColor = Options.titleTextColor;
		if (Options.frameWidth != null) frameWidth = Options.frameWidth;
		if (Options.navPadding != null) navPadding = Options.navPadding;
		if (Options.thumbnailColumns != null) thumbnailColumns = Options.thumbnailColumns;
		if (Options.thumbnailRows != null) thumbnailRows = Options.thumbnailRows;
		if (Options.navPosition != null) navPosition = Options.navPosition;
		if (Options.imagePath != null) imagePath = Options.imagePath;
		if (Options.thumbPath != null) thumbPath = Options.thumbPath;
		if (Options.title != null) title = Options.title;
		if (Options.enableRightClickOpen != null) enableRightClickOpen = Options.enableRightClickOpen;
		if (Options.backgroundImagePath != null) backgroundImagePath = Options.backgroundImagePath;
		if (Options.vAlign != null) vAlign = Options.vAlign;
		if (Options.hAlign != null) hAlign = Options.hAlign;
							
		//get image data																			
		imageCount = Number(xmlRoot.childNodes.length);
		for (var i:Number = 0;i<imageCount;i++){											
			var resultNode:XMLNode = xmlRoot.childNodes[i];	
			imageFileNames.push(resultNode.childNodes[0].firstChild.nodeValue);																				
			imageCaptions.push(resultNode.childNodes[1].firstChild.nodeValue);																											
		}	
		
		//render results
		mStageManager.onXMLLoaded();		
	}
		
	function setIfValidNumber(xmlVal:String,defaultVal:Number){			
		var v = Number(xmlVal);
		if (isValidNumber(v)){
			return v;
		}else{
			return defaultVal;
		}				
	}
	
	function isValidNumber(x:Number){		
		return !(isNaN(x) || x == undefined);
		
	}
	
	function isValidPath(s:String){
		return !(s == "" || s == undefined);
	}
		
}
